#!/usr/bin/env python
#
# Solve 2x2 linear equation system of 2 - with GUI by Gtk
# Author:  Yotam Medini  yotam.medini@gmail.com -- Created: 2007/February/23
#

import gtk
import sys

# local
import lineq2


class GLineEq2:

    def __init__(self):
        self.build_gui()
        self.solver = lineq2.LineEq2x2()


    def build_gui(self):
        self.main_window = w = gtk.Window(gtk.WINDOW_TOPLEVEL)
        w.connect("destroy", lambda w: gtk.main_quit())
        w.set_title("Linear 2x2 Equations Solver")
        vbox = gtk.VBox(False, 12)
        title = gtk.Label("""
Please enter coefficients
as integers or fractions
"""[1:])
        title.show()
        vbox.pack_start(title)

        self.coefs = 6*[None]
        for eqi in 0, 1:
            vbox.pack_start( self.build_equation_box(eqi) )

        self.result = gtk.Label("")
        # self.result.hide()
        self.result.show()
        vbox.pack_start(self.result)
        vbox.pack_start( self.build_action_box() )
        
        vbox.show()
        w.add(vbox)
        w.show()


    def build_equation_box(self, eqi):
        hbox = gtk.HBox()
        abc = 3*[None]
        for ei in 0,1,2:
            abc[ei] = e = gtk.Entry()
            e.connect("key_press_event", self.change_call)
            self.coefs[3*eqi + ei] = e
        hbox.pack_start(abc[0], padding=6)  # a
        hbox.pack_start(gtk.Label("x   +  "))
        hbox.pack_start(abc[1], padding=6)  # b
        hbox.pack_start(gtk.Label("y   =  "))
        hbox.pack_start(abc[2], padding=6)  # c
        hbox.show_all() 
        return hbox


    def build_action_box(self):
        hbox = gtk.HBox()
        hbox.set_homogeneous(True)
        solve = gtk.Button("Solve")
        close = gtk.Button(stock=gtk.STOCK_CLOSE)
        hbox.pack_start(solve, fill=False)
        hbox.pack_start(close, fill=False)
        hbox.show_all()

        solve.connect("clicked", self.solve)
        close.connect("clicked", lambda w: gtk.main_quit())
        return hbox
        

    def change_call(self, w, dummy=None):
        self.result.set_text("")


    def solve(self, w, dummy=None):
        solver = self.solver; # abbreviate
        s = map(lambda e: e.get_text(), self.coefs) # list of entries strings
        err = solver.set_coefficients(s)
        if err != None:
            # sys.stderr.write("error: %s\n" % err)
            warn = gtk.MessageDialog(
                self.main_window,
                gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                gtk.MESSAGE_ERROR,
                gtk.BUTTONS_OK,
                err)
            warn.set_title("Error: Line Equations Solver")
            warn.run()
            warn.destroy()
        else:
            leq_rc = solver.solve()
            msg = lineq2.name[leq_rc]
            if leq_rc in (lineq2.UniqueSolution, lineq2.InfiniteSolutions):
                msg += ":   x = %s   y = %s" % (solver.x, solver.y)
            self.result.set_text(msg)


rc = 0
g = GLineEq2()
gtk.main()
sys.exit(rc)
